use tokio::{io::{AsyncReadExt, AsyncWriteExt}, net::TcpListener};
use std::net::SocketAddr;
use crate::{message::Message, message_read::MessageReader};

pub struct Server {
    pub host: String,
    pub port: u16,
}

impl Server {
    pub fn new(host: impl Into<String>, port: u16) -> Self {
        Server {
            host: host.into(),
            port,
        }
    }

    pub async fn run(&mut self) -> anyhow::Result<()> {
        let listener = TcpListener::bind(format!(
            "{}:{}",
            self.host,
            self.port
        )).await?;

        println!("Server is running on {}:{}", self.host, self.port);

        loop {
            let (mut socket, addr) = listener.accept().await?;
            println!("Connection received from {}", addr);

            tokio::task::spawn(async move {
                let mut message_reader = MessageReader::new();

                let (mut rd, mut wr) = socket.split();

                'handler: loop {
                    let mut buffer = vec![];
                    if tokio::io::copy(&mut rd, &mut buffer).await.is_err() {
                        eprintln!("failed to copy");
                    }
                    
                    let message =  message_reader.read(&buffer)?;
                    println!("{:?}", message.content);

                    break 'handler;
                }

                Ok::<(), anyhow::Error>(())
            });
        }

        Ok(())
    }

    pub async fn send_message(addr: SocketAddr, message: Message) {
        todo!()
    }
}
